/*
 * KernelConf - A Linux Kernel Configuration System
 *
 * Distributed under GNU General Public Lisence
 *
 * Copyright (C) 2002 by Anuradha Ratnaweera
 */

#ifndef _KCONF_SYMBOLS_H
#define _KCONF_SYMBOLS_H

#include "tree.h"
#include "expressions.h"
#include "values.h"

typedef struct _ConfFile* ConfFilePtr;

typedef struct _ConfFile {
    char *name;
    ConfFilePtr next;
} ConfFile;

typedef struct _Symbol* SymbolPtr;
typedef struct _Menu* MenuPtr;

/* 
 * Both Symbol and Menu are derived from Node structure.  To access
 * an element's name, left, right, next and flags, it should be cast
 * to Node or NodePtr.
 */

typedef struct _Symbol {
    Node node;
    Type type;
    Value value;
    ExpressionPtr depends;
    ExpressionPtr derived;
    ExpressionPtr affects;
    MenuPtr menu_item;
    ConfFilePtr conf_file;
    long desc_start, desc_length;
} Symbol;

typedef struct _Menu {
    Node node;
    MenuPtr parent, child;
    char *caption;
    ExpressionPtr depends;
    SymbolPtr symbol;
    ConfFilePtr conf_file;
    long desc_start, desc_length;
} Menu;

long read_help(MenuPtr, char**);

#endif /* _KCONF_SYMBOLS_H */

