/*
 * KernelConf - A Linux Kernel Configuration System
 *
 * Distributed under GNU General Public Lisence
 *
 * Copyright (C) 2002 by Anuradha Ratnaweera
 */

#include <malloc.h>

#include "tree.h"

static NodePtr new_node(char *key, size_t size)
{
    NodePtr tmp;

    tmp = (NodePtr)malloc(size);
    tmp->name = (char*)malloc(sizeof(char) * (strlen(key) + 1));
    strcpy(tmp->name, key);
    tmp->left = NULL;
    tmp->right = NULL;
    tmp->next = NULL;

    return (tmp);
}

/*
 * looks for the string KEY in the tree pointed by ROOT and if
 * it is not found, inserts to the tree if INSERT is set to 
 * NODE_INSERT or NODE_MARK_INSERT. Returns NULL if INSERT is
 * set to NODE_SEARCH.  In NODE_MARK_INSERT case, the NODE_FLAG_MARK
 * is set to indicate that the node is not initialized.  This flag
 * is cleared in a later NODE_INSERT request for the same key.
 *
 * SIZE is the size of the structure derived from Node.
 */
NodePtr search_tree(NodePtr *root, char *key, int insert, size_t size, NodePtr last)
{
    NodePtr ptr, balance, parent, new;
    int cmp;

    if ((*root) == NULL) {
	if (insert == NODE_SEARCH)
	    return NULL;
	new = new_node(key, size);
	if (insert == NODE_MARK_INSERT)
	    new->flags = NODE_FLAGS_MARK;
	else
	    new->flags = 0;
	*root = new;
	if (last)
	    last->next = new;
	return new;
    }

    ptr = *root;
    balance = *root;
    parent = *root;

    while (1) {
	cmp = strcmp(key, ptr->name);

	if (cmp < 0) {
	    if (ptr->left) {
		if ((ptr->flags & NODE_FLAGS_BF) - 1) {
		    parent = ptr;
		    balance = ptr->left;
		}
		ptr = ptr->left;
	    }
	    else {
		if (insert == NODE_SEARCH)
		    return NULL;
		new = new_node(key, size);
		ptr->left = new;
		break;
	    }
	}
	else if (cmp > 0) {
	    if (ptr->right) {
		if ((ptr->flags & NODE_FLAGS_BF) - 1) {
		    parent = ptr;
		    balance = ptr->right;
		}
		ptr = ptr->right;
	    }
	    else {
		if (insert == NODE_SEARCH)
		    return NULL;
		new = new_node(key, size);
		ptr->right = new;
		break;
	    }
	}
	else {
	    /* found */
	    if ((insert == NODE_INSERT) && (ptr->flags & NODE_FLAGS_MARK))
		ptr->flags -= NODE_FLAGS_MARK;
	    return ptr;
	}
    }

    /* we have just inserted a new node at NEW */
    if (insert == NODE_MARK_INSERT)
	new->flags = NODE_FLAGS_MARK;
    else
	new->flags = 0;

    if (last)
	last->next = new;

    return new;

    /* FIXME: balance the tree */

    return NULL;
}

