/*
 * KernelConf - A Linux Kernel Configuration System
 *
 * Distributed under GNU General Public Lisence
 *
 * Copyright (C) 2002 by Anuradha Ratnaweera
 */

#include <stdio.h>

#include "symbols.h"
#include "writeconfig.h"

extern SymbolPtr first_symbol;

int write_config(char *conf_filename, char *conf_h_filename)
{
    FILE *conf, *conf_h;
    SymbolPtr ptr;

    if ((conf= fopen(conf_filename, "wt")) == NULL) {
	exit(1);
    }

    if ((conf_h= fopen(conf_h_filename, "wt")) == NULL) {
	exit(1);
    }

    ptr = first_symbol;

    fprintf(conf_h, "#define AUTOCONF_INCLUDED\n");

    while (ptr) {
	switch (ptr->type) {
	    case TYPE_TRI:
	    case TYPE_BOOL:
	    case TYPE_OPTION:
		if (ptr->value.charval == 'y') {
		    fprintf(conf_h, "#define CONFIG_%s 1\n",
			    ((NodePtr)ptr)->name);
		    fprintf(conf, "CONFIG_%s=y\n", ((NodePtr)ptr)->name);
		}
		else if (ptr->value.charval == 'm') {
		    fprintf(conf_h, "#undef  CONFIG_%s\n", ((NodePtr)ptr)->name);
		    fprintf(conf_h, "#define CONFIG_%s_MODULE 1\n",
			    ((NodePtr)ptr)->name);
		    fprintf(conf, "CONFIG_%s=m\n", ((NodePtr)ptr)->name);
		}
		else {
		    fprintf(conf_h, "#undef  CONFIG_%s\n",
			    ((NodePtr)ptr)->name);
		    fprintf(conf, "# CONFIG_%s is not set\n", ((NodePtr)ptr)->name);
		}
	    case TYPE_CHAR:
		break;
	    case TYPE_NULL:
		fprintf(conf_h, "#undef  CONFIG_%s\n", ((NodePtr)ptr)->name);
		fprintf(conf, "# CONFIG_%s is not set\n", ((NodePtr)ptr)->name);
		break;
	    case TYPE_INT:
	    case TYPE_HEX:
		break;
	    case TYPE_STR:
		break;
	    default:
		;
	}

	ptr = (SymbolPtr)((NodePtr)ptr)->next;
    }

    fclose(conf);
    fclose(conf_h);

    return 0;
}
